/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.test;

import javax.jms.Connection;
import javax.jms.JMSException;
import javax.jms.Session;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.Topic;

import org.activemq.ActiveMQConnectionFactory;
import org.activemq.spring.SpringBrokerContainerFactory;
import org.springframework.core.io.ClassPathResource;
import org.activemq.message.ActiveMQDestination;
import org.activemq.broker.BrokerContainer;
import org.activemq.service.DeadLetterPolicy;
import org.activemq.service.MessageIdentity;
import org.activemq.store.RecoveryListener;
import org.activemq.store.jdbc.JDBCPersistenceAdapter;
import org.activemq.store.jdbc.PersistenceAdapterTest;

/**
 * @author Ramzi Saba
 * 
 */
public class DeadLetterManualExpiryTest extends TestSupport {
    
    private static final int MESSAGE_COUNT = 10;
    private static final long TIME_TO_LIVE = 250;
    private static final long CLEANUP_REPEAT_INTERVAL = 1000*2; // auto-cleanup every 2 seconds
    private ActiveMQConnectionFactory factory;
    private Connection connection;
    private Session session;
    private MessageConsumer consumer;
    private MessageProducer producer;
    private ActiveMQDestination destination;
    private int deliveryMode = DeliveryMode.PERSISTENT;
    private boolean durableSubscriber = false;
    private static String BROKER_CONFIG = "org/activemq/spring/activemq.xml";
    private BrokerContainer brokerContainer;
    private JDBCPersistenceAdapter adapter;
    
    protected boolean autoExpire = false;
    protected boolean expirePersistentMessagesOnly = false;
   
    protected void setUp() throws Exception{
        super.setUp();
        brokerContainer = createBrokerContainer();
        brokerContainer.start();
        factory = createConnectionFactory();
        connection = factory.createConnection();
        connection.setClientID(toString());        
        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
    }
    
    protected BrokerContainer createBrokerContainer() {
    	BrokerContainer container = SpringBrokerContainerFactory.newInstance(new ClassPathResource(BROKER_CONFIG), "broker");
        adapter = (JDBCPersistenceAdapter)PersistenceAdapterTest.createDefault(true);
        adapter.setAutoCleanupExpiredMessages(autoExpire);
        adapter.setCleanupRepeatInterval(CLEANUP_REPEAT_INTERVAL);
        adapter.setDeleteExpiredMessages(true);
        adapter.setBrokerContainer(container);
        container.setPersistenceAdapter(adapter);
        return container;
    }
    
    protected ActiveMQConnectionFactory createConnectionFactory() {
        ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory(brokerContainer, "tcp://localhost:61626");
        factory.setUseEmbeddedBroker(false);
        return factory;
    }
    
    protected void tearDown() throws Exception {
        connection.close();
        brokerContainer.stop();
    }
    
    protected void doTest() throws Exception{
        produceAndConsumeMessages();
        // DLQ is not enabled for transient topics
        if (!topic || deliveryMode == DeliveryMode.PERSISTENT) {
        	verifyDeadLetterQueue();
        }
    }
    
    protected void produceAndConsumeMessages () throws JMSException, InterruptedException {
    	String destinationName = getClass().getName();
        destination = (ActiveMQDestination) createDestination(destinationName);
        producer = session.createProducer(destination);
        producer.setDeliveryMode(deliveryMode);
        producer.setTimeToLive(TIME_TO_LIVE);

        if (durableSubscriber){
            consumer = session.createDurableSubscriber((Topic)destination, destination.toString());
        }else {
            consumer = session.createConsumer(destination);
        }
        
        for (int i =0; i < MESSAGE_COUNT;i++){
            Message message = session.createTextMessage("msg: " + i);
            producer.send(message);
        }
        Thread.sleep(TIME_TO_LIVE * 2);
        connection.start();
        
        if (autoExpire) {
        	Thread.sleep(CLEANUP_REPEAT_INTERVAL*2);
        	// expired messages should have been deleted by now
            adapter.createQueueMessageStore(destinationName).recover(new RecoveryListener() {
            	public void recoverMessage(MessageIdentity messageIdentity) throws JMSException {
            		assertTrue("Queue should be empty, found message: " + messageIdentity, false);
            	}
            });        	
        } else {
	        for (int i =0; i < MESSAGE_COUNT; i++){
	            Message msg = consumer.receive(10);
	            assertNull("Should be null message", msg);
	        }	        
        }
    }
    
    protected void verifyDeadLetterQueue() throws JMSException, InterruptedException {
        DeadLetterPolicy dlq = brokerContainer.getBroker().getDeadLetterPolicy();
        String dlqName = dlq.getDeadLetterNameFromDestination(destination);
        Destination dlqDestination = session.createQueue(dlqName);
        MessageConsumer dlqConsumer = session.createConsumer(dlqDestination); 
        
        for (int i = 0; i < MESSAGE_COUNT; i++){
            Message msg = dlqConsumer.receive(1000);
            assertNotNull("Should find message in dead letter queue", msg);
        }
        
        // wait for consumed messages to be removed from the store
        Thread.sleep(2000);
        
    	// dead letter queue should be clean by now
        adapter.createQueueMessageStore(dlqName).recover(new RecoveryListener() {
        	public void recoverMessage(MessageIdentity messageIdentity) throws JMSException {
        		assertTrue("Dead letter queue should be empty, found message: " + messageIdentity, false);
        	}
        });
    }
    
    public void testTransientTopicMessageExpiration() throws Exception{
        super.topic = true;
        deliveryMode = DeliveryMode.NON_PERSISTENT;
        durableSubscriber = false;
        doTest();
    }
    
    public void testDurableTopicMessageExpiration() throws Exception{
        super.topic = true;
        deliveryMode = DeliveryMode.PERSISTENT;
        durableSubscriber = true;
        doTest();
    }
    
    public void testTransientQueueMessageExpiration() throws Exception{
    	if (!expirePersistentMessagesOnly) {
	        super.topic = false;
	        deliveryMode = DeliveryMode.NON_PERSISTENT;
	        durableSubscriber = false;
	        doTest();
    	}
    }
    
    public void testDurableQueueMessageExpiration() throws Exception{
        super.topic = false;
        deliveryMode = DeliveryMode.PERSISTENT;
        durableSubscriber = false;
        doTest();
    }
   
}
